#version 330 core

// Interpolated values from the vertex shaders
in vec3 PositionWorldPass;
in vec3 NormalWorldPass;
in vec2 TexCoordPass0;
in vec2 TexCoordPass1;

// Ouput data
out vec4 Color;

// Values that stay constant for the whole mesh.
uniform int gModulate;
uniform sampler2D gTextureSampler1;
uniform sampler2D gTextureSampler2;
uniform mat4 gModel, gView, gProjection;
uniform vec3 gEyePosition; // camera position (in world)

struct Light
{
	vec4 Position;
	vec4 Ambient;
	vec4 Diffuse;
	vec4 Specular;

	float ConstantAttenuation;
	float LinearAttenuation;
	float QuadraticAttenuation;

	float SpotCutoff;
	float SpotExponent;
	vec3 SpotDirection;
};
const int numberOfLights = 3;
Light lights[numberOfLights];
uniform Light gLight0;
uniform Light gLight1;
uniform Light gLight2;

struct Material
{
	vec4 Ambient;
	vec4 Diffuse;
	vec4 Specular;
	float Shininess;
};
uniform Material gMaterial;

void main()
{
	lights[0] = gLight0;
	lights[1] = gLight1;
	lights[2] = gLight2;

	vec3 normalDirection = normalize(NormalWorldPass);
	vec3 viewDirection = normalize(gEyePosition - PositionWorldPass);

	vec3 lightDirection;
	float attenuation;
	float lightPower = 1.0f;
	vec3 totalLighting = vec3(0.2, 0.2, 0.2) * vec3(gMaterial.Ambient);

	// initialize total lighting with ambient lighting
	for (int index = 0; index < numberOfLights; index++) // for all light sources
	{
		if (0.0 == lights[index].Position.w) // directional light?
		{
			attenuation = 1.0; // no attenuation
			lightDirection = normalize(vec3(lights[index].Position));
		} 
		else // point light or spotlight (or other kind of light) 
		{
			vec3 vertexToLightSource = lights[index].Position.xyz - PositionWorldPass;
			float distance = length(vertexToLightSource);
			lightDirection = normalize(vertexToLightSource);
			attenuation = 1.0 / (lights[index].ConstantAttenuation
			       + lights[index].LinearAttenuation * distance
			       + lights[index].QuadraticAttenuation * distance * distance);
	  
			if (lights[index].SpotCutoff <= 90.0) // spotlight?
			{
				float clampedCosine = max(0.0, dot(-lightDirection, normalize(lights[index].SpotDirection)));
				if (clampedCosine < cos(radians(lights[index].SpotCutoff))) // outside of spotlight cone?
				{
					attenuation = 0.0;
				}
				else
				{
					attenuation = attenuation * pow(clampedCosine, lights[index].SpotExponent);   
				}
			}
		}

		// Material properties
		vec4 materialDiffuse;
		if (gModulate == 1)
			materialDiffuse = texture2D(gTextureSampler1, TexCoordPass0).rgba * texture2D(gTextureSampler2, TexCoordPass1).rgba; // modulate
		else
			materialDiffuse = texture2D(gTextureSampler1, TexCoordPass0).rgba + texture2D(gTextureSampler2, TexCoordPass1).rgba; // add

		materialDiffuse = materialDiffuse * gMaterial.Diffuse;
		
		vec3 diffuseReflection = attenuation 
								* vec3(lights[index].Diffuse) * vec3(materialDiffuse)
								* max(0.0, dot(normalDirection, lightDirection));
      
		vec3 specularReflection;
		if (dot(normalDirection, lightDirection) < 0.0) // light source on the wrong side?
		{
			specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
		}
		else // light source on the right side
		{
			specularReflection = attenuation * vec3(lights[index].Specular) * vec3(gMaterial.Specular) 
								* pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), gMaterial.Shininess);
		}

		totalLighting = totalLighting + diffuseReflection + specularReflection;
	}
  
	Color = vec4(totalLighting, 1.0);
}
